: # Use perl
eval 'exec perl -S $0 "$@"'
  if $running_under_some_shell;

use strict;
use Getopt::Std;
use vars qw(%opt %wx %llx %urx @bbox @char $usage $opts $name $start $str
	    $check $wid $i $x $y $size $Y);

$opts = "hp";
$usage =<<EOD;

Usage: code128.pl [-$opts] [string]

This script converts a string into the correct form to be printed as a Code 128
bar code, using the Code128JK font.

Options:
  -h  Print this help message
  -p  Generate PostScript - printed to standard output
  
The parameter is the message string to be converted, when absent the script
will prompt for the string.
EOD

getopts($opts, \%opt) or die "$usage\n";

die "$usage\n" if ($opt{h});

use vars qw(%opt %wx %llx %urx @bbox @char $name $start $str $check $wid $i
	    $x $y $size $Y);

if ($#ARGV == -1) {
    print STDERR "Enter message to be converted: ";
    chop($ARGV[0]=<STDIN>);
};

undef $/;

$x = 50;
$y = 50;
$size = 40;
$name = "Code128JK";

my ($starta, $startb, $startc, $codea, $codeb, $codec)
    = ("\207", "\210", "\211", "\205", "\204", "\203");

die "Invalid character: $&\n" if ($ARGV[0] =~ /[\200-\377]/);

# Use Code C where there are at least four consecutive integers

my @temp = split /((?:\d\d){2,})/, $ARGV[0];
$str = $temp[0];
for $i (1..$#temp) {
    if ($i % 2) {
	$str .= $str eq "" ? $startc : $codec;
	$temp[$i] =~ s/\d\d/chr($&+32)/eg;
    } else {
	$str .= $temp[$i] =~ /^[\000-\037]/ ? $codea : $codeb;
    }
    $str .= $temp[$i];
}

# Insert appropriate start character (Code A and B)

if ($str =~ /^[\000-\037]/) {
    $str = $starta . $str;
} elsif ($str =~ /^[ -\177]/) {
    $str = $startb . $str;
}

# Insert code shift characters when necessary (between control characters
# and non-control characters)

$str =~ s/([\000-\037])([ -\177])/$1$codeb$2/g;
$str =~ s/([ -\177])([\000-\037])/$1$codea$2/g;

# Shift control characters

$str =~ tr/\000-\037/`-\177/;

# Generate the check character

@char = split //, $str;
$check = ord($char[0]) - 32;
for $i (1..$#char) {
    $check += (ord($char[$i]) - 32)*$i;
}

# Insert check and stop character

$str .= chr($check % 103 + 32) . chr(138);
$str =~ y/\177-\212/\300-\313/;

if (!$opt{p}) {
    print "$str\n";
    exit;
}

open AFM, "../../pfa/$name.afm" or die "Cannot open font metric file\n";
$_ = <AFM>;
close AFM;
($Y) = /FontBBox.* (\d+) *$/m;
while (/C +(\d+) *; *WX +(\d+) *; *N +\S+ *; *B +(\d+) +\d+ +(\d+)/g) {
    $wx{$1} = $2;
    $llx{$1} = $3;
    $urx{$1} = $4;
}

@char = split //, $str;
$wid = $urx{ord pop @char};
for (@char) {
    $wid += $wx{ord $_};
}
@bbox = (int($x + $llx{ord $char[0]}*$size/1000), int($y),
    int($x + $wid*$size/1000 + .999), int($y + $Y*$size/1000 + .999));

$str =~ s/[)(\\]/\\$&/g;
$str =~ s/[\000-\037\177-\240]/sprintf "\\%3.3o", ord $&/eg;

open PFA, "../../pfa/$name.pfa" or die "Cannot open font file\n";
$_ = <PFA>;
close PFA;

print <<EOD;
%!PS-Adobe-3.0
%%BoundingBox: @bbox
%%Pages: 1
%%EndComments
%%BeginSetup
%%BeginResource: font $name
$_
%%EndResource
%%EndSetup

%%Page: 1 1
/$name findfont $size scalefont setfont
$x $y moveto
($str)show
showpage
%%EOF
EOD
