: # Use perl
eval 'exec perl -S $0 "$@"'
  if $running_under_some_shell;

use strict;
use Getopt::Std;
use vars qw(%opt %wx %llx %urx @bbox @char $usage $opts $name $len $start $str
	    $check $wid $i $x $y $size $Y);

$opts = "ehp";
$usage =<<EOD;

Usage: ean.pl [-$opts] [string [string]]

This script converts a message into the correct form to be printed as one of
the UPC A, UPC E, EAN 13 or EAN 8 bar codes, using the EANJK font.

Options:
  -e  Generate a UPC-E bar code
  -h  Print this help message
  -p  Generate PostScript - printed to standard output
  
The parameters are the message strings to be converted, when absent the script
will prompt for the message. The first string is the UPC/EAN message, and the
optional second is the two or five digit supplemental. The last digit in the
final UPC/EAN message, the checksum character, should be omitted from the
string. So, for example, an EAN 13 message should be given as a 12 digit
number, and a UPC A message should have 11 digits.
EOD

getopts($opts, \%opt) or die "$usage\n";

die "$usage\n" if ($opt{h});

if ($#ARGV == -1) {
    print "Enter message to be converted: ";
    @ARGV = split /\s+/, <STDIN>;
};

undef $/;

$x = 50;
$y = 50;
$size = 40;
$name = "EANJK";

die "$usage\n" if($#ARGV < 0 || $#ARGV > 1);

my @parity = (
[1, 1, 1, 1, 1, 1],
[1, 1, 0, 1, 0, 0],
[1, 1, 0, 0, 1, 0],
[1, 1, 0, 0, 0, 1],
[1, 0, 1, 1, 0, 0],
[1, 0, 0, 1, 1, 0],
[1, 0, 0, 0, 1, 1],
[1, 0, 1, 0, 1, 0],
[1, 0, 1, 0, 0, 1],
[1, 0, 0, 1, 0, 1]
);

my @parity2 = @parity;
$parity2[0] = [1, 1, 1, 0, 0, 0];

die "Illegal character: $&\n" if ($ARGV[0] =~ /\D/ || $ARGV[1] =~ /\D/);

if ($ARGV[0] =~ /^(.)(.)(.)(.)(.)(.)$/) {
    $opt{e} = 1;
    SW: {
	$6  < 3 and $ARGV[0] = "0$1$2$6" . "0000$3$4$5", last SW;
	$6 == 3 and $ARGV[0] = "0$1$2$3" . "00000$4$5", last SW;
	$6 == 4 and $ARGV[0] = "0$1$2$3$4" . "00000$5", last SW;
	$ARGV[0] = "0$1$2$3$4$5" . "0000$6";
    }
}
chop $ARGV[0] if (length $ARGV[0] == 8 || length $ARGV[0] == 13);
$ARGV[0] = "0$ARGV[0]" if (length $ARGV[0] == 11);
$len = length $ARGV[0];
die "Invalid UPC/EAN message\n" if ($len != 7 && $len != 12);
@char = split //, $ARGV[0];
if ($len == 12) {
    $start = shift @char;
    $check = $start;
}
$str = "";
my $weight = 3;
for $i (0..$#char) {
    if ($len == 12) {
	if ($i < 6) {
	    $str .= $parity[$start][$i] ? $char[$i] : chr($char[$i]+97);
	} else {
	    $str .= chr($char[$i]+65);
	}
	$str .= "-" if ($i == 5);
    } elsif ($len == 7) {
	$str .= $i < 4 ? $char[$i] : chr($char[$i]+65);
	$str .= "-" if ($i == 3);
    }
    $check += $weight*$char[$i];
    $weight = 4 - $weight;
}

if ($opt{e}) {
    my $temp;
    SW: {
	$ARGV[0] =~ /^0.(..)([0-2])0{4}(.+)/ and $temp = "$1$3$2",     last SW;
	$ARGV[0] =~ /^0.(..[3-9])0{5}(.+)/   and $temp = "$1$2" . "3", last SW;
	$ARGV[0] =~ /^0.(...[1-9])0{5}(.)/   and $temp = "$1$2" . "4", last SW;
	$ARGV[0] =~ /^0.(....[1-9])0{4}(.)/  and $temp = "$1$2",       last SW;
	die "UPC-E cannot be generated from this number\n";
    }
    @char = split //, $temp;
    $str = "|";
    for $i (0..5) {
	$str .= $parity2[-$check % 10][$i] ? chr($char[$i]+97) : $char[$i];
    }
    $str .= "}";
} else {
    $str = "|$str" . chr(-$check % 10 + 65) . "|";
}

if ($#ARGV == 1) {
    @char = split //, $ARGV[1];
    $str .= " [";
    if ($#char == 1) {
	$str .= ($ARGV[1] % 4 < 2 ? $char[0] : chr($char[0]+97)) . ":";
	$str .= $ARGV[1] % 2 == 0 ? $char[1] : chr($char[1]+97);
    } elsif ($#char == 4) {
	$check = 0;
	for $i (0..$#char) {
	    $check += 3*$char[$i]*(1 + 2*($i % 2));
	}
	for (0..4) {
	    $str .= $parity2[$check % 10][$_+1] == 0 ? $char[$_] : chr($char[$_]+97);
	    $str .= ":" if ($_ < 4);
	}
    } else {
	die "Supplemental must consist of 2 or 5 digits\n";
    }
}

if (!$opt{p}) {
    print "$str\n";
    exit;
}

open AFM, "../../pfa/$name.afm" or die "Cannot open font metric file\n";
$_ = <AFM>;
close AFM;
($Y) = /FontBBox.* (\d+) *$/m;
while (/C +(\d+) *; *WX +(\d+) *; *N +\S+ *; *B +(\d+) +\d+ +(\d+)/g) {
    $wx{$1} = $2;
    $llx{$1} = $3;
    $urx{$1} = $4;
}

@char = split //, $str;
$wid = $urx{ord pop @char};
for (@char) {
    $wid += $wx{ord $_};
}
@bbox = (int($x + $llx{ord $char[0]}*$size/1000), int($y),
    int($x + $wid*$size/1000 + .999), int($y + $Y*$size/1000 + .999));

$str =~ s/[)(\\]/\\$&/g;
$str =~ s/[\000-\037\177-\240]/sprintf "\\%3.3o", ord $&/eg;

open PFA, "../../pfa/$name.pfa" or die "Cannot open font file\n";
$_ = <PFA>;
close PFA;

print <<EOD;
%!PS-Adobe-3.0
%%BoundingBox: @bbox
%%Pages: 1
%%EndComments
%%BeginSetup
%%BeginResource: font $name
$_
%%EndResource
%%EndSetup

%%Page: 1 1
/$name findfont $size scalefont setfont
$x $y moveto
($str)show
showpage
%%EOF
EOD
